@@ -102,10 +102,10 @@ struct fio_file {
 	/*
 	 * if io is protected by a semaphore, this is set
 	 */
-	struct fio_mutex *lock;
-	void *lock_owner;
-	unsigned int lock_batch;
-	enum fio_ddir lock_ddir;
+	union {
+		struct fio_mutex *lock;
+		struct fio_rwlock *rwlock;
+	};
 
 	/*
 	 * block map for random io
@@ -176,5 +176,6 @@ extern void dup_files(struct thread_data *, struct thread_data *);
 extern int get_fileno(struct thread_data *, const char *);
 extern void free_release_files(struct thread_data *);
 void fio_file_reset(struct thread_data *, struct fio_file *);
+int fio_files_done(struct thread_data *);
 
 #endif
