@@ -100,7 +100,7 @@ static struct option l_opts[FIO_NR_OPTIONS] = {
 	},
 	{
 		.name		= (char *) "minimal",
-		.has_arg	= optional_argument,
+		.has_arg	= no_argument,
 		.val		= 'm' | FIO_CLIENT_FLAG,
 	},
 	{
@@ -207,6 +207,11 @@ static struct option l_opts[FIO_NR_OPTIONS] = {
 		.has_arg	= no_argument,
 		.val		= 'T',
 	},
+	{
+		.name		= (char *) "crctest",
+		.has_arg	= optional_argument,
+		.val		= 'G',
+	},
 	{
 		.name		= (char *) "idle-prof",
 		.has_arg	= required_argument,
@@ -245,6 +250,7 @@ void free_shm(void)
 		free_threads_shm();
 	}
 
+	options_free(fio_options, &def_thread);
 	scleanup();
 }
 
@@ -635,6 +641,12 @@ static int fixup_options(struct thread_data *td)
 	if (td->o.oatomic)
 		td->o.odirect = 1;
 
+	/*
+	 * If randseed is set, that overrides randrepeat
+	 */
+	if (td->o.rand_seed)
+		td->o.rand_repeatable = 0;
+
 	return ret;
 }
 
@@ -830,10 +842,15 @@ static int setup_random_seeds(struct thread_data *td)
 	unsigned long seed;
 	unsigned int i;
 
-	if (!td->o.rand_repeatable)
+	if (!td->o.rand_repeatable && !td->o.rand_seed)
 		return init_random_state(td, td->rand_seeds, sizeof(td->rand_seeds));
 
-	for (seed = 0x89, i = 0; i < 4; i++)
+	if (!td->o.rand_seed)
+		seed = 0x89;
+	else
+		seed = td->o.rand_seed;
+
+	for (i = 0; i < 4; i++)
 		seed *= 0x9e370001UL;
 
 	for (i = 0; i < FIO_RAND_NR_OFFS; i++) {
@@ -1146,9 +1163,9 @@ void add_job_opts(const char **o, int client_type)
 			td = get_new_job(0, td_parent, 0);
 		}
 		if (in_global)
-			fio_options_parse(td_parent, (char **) &o[i], 1);
+			fio_options_parse(td_parent, (char **) &o[i], 1, 0);
 		else
-			fio_options_parse(td, (char **) &o[i], 1);
+			fio_options_parse(td, (char **) &o[i], 1, 0);
 		i++;
 	}
 
@@ -1329,14 +1346,10 @@ int parse_jobs_ini(char *file, int is_buf, int stonewall_flag, int type)
 			num_opts++;
 		}
 
-		ret = fio_options_parse(td, opts, num_opts);
-		if (!ret) {
-			if (dump_cmdline)
-				for (i = 0; i < num_opts; i++)
-					log_info("--%s ", opts[i]);
-
+		ret = fio_options_parse(td, opts, num_opts, dump_cmdline);
+		if (!ret)
 			ret = add_job(td, name, 0, 0, type);
-		} else {
+		else {
 			log_err("fio: job %s dropped\n", name);
 			put_job(td);
 		}
@@ -1386,7 +1399,7 @@ static void usage(const char *name)
 	printf("%s [options] [job options] <job file(s)>\n", name);
 	printf("  --debug=options\tEnable debug logging. May be one/more of:\n"
 		"\t\t\tprocess,file,io,mem,blktrace,verify,random,parse,\n"
-		"\t\t\tdiskutil,job,mutex,profile,time,net\n");
+		"\t\t\tdiskutil,job,mutex,profile,time,net,rate\n");
 	printf("  --parse-only\t\tParse options only, don't start any IO\n");
 	printf("  --output\t\tWrite output to file\n");
 	printf("  --runtime\t\tRuntime in seconds\n");
@@ -1398,6 +1411,7 @@ static void usage(const char *name)
 	printf("  --version\t\tPrint version info and exit\n");
 	printf("  --help\t\tPrint this page\n");
 	printf("  --cpuclock-test\tPerform test/validation of CPU clock\n");
+	printf("  --crctest\t\tTest speed of checksum functions\n");
 	printf("  --cmdhelp=cmd\t\tPrint command help, \"all\" for all of"
 		" them\n");
 	printf("  --enghelp=engine\tPrint ioengine help, or list"
@@ -1425,7 +1439,8 @@ static void usage(const char *name)
 		"\t\t\t(option=system,percpu) or run unit work\n"
 		"\t\t\tcalibration only (option=calibrate)\n");
 	printf("\nFio was written by Jens Axboe <jens.axboe@oracle.com>");
-	printf("\n                   Jens Axboe <jaxboe@fusionio.com>\n");
+	printf("\n                   Jens Axboe <jaxboe@fusionio.com>");
+	printf("\n                   Jens Axboe <axboe@fb.com>\n");
 }
 
 #ifdef FIO_INC_DEBUG
@@ -1486,6 +1501,10 @@ struct debug_level debug_levels[] = {
 	  .help = "Network logging",
 	  .shift = FD_NET,
 	},
+	{ .name = "rate",
+	  .help = "Rate logging",
+	  .shift = FD_RATE,
+	},
 	{ .name = NULL, },
 };
 
@@ -1592,6 +1611,8 @@ void parse_cmd_client(void *client, char *opt)
 	fio_client_add_cmd_option(client, opt);
 }
 
+extern int fio_crctest(const char *);
+
 int parse_cmd_line(int argc, char *argv[], int client_type)
 {
 	struct thread_data *td = NULL;
@@ -1640,6 +1661,12 @@ int parse_cmd_line(int argc, char *argv[], int client_type)
 			output_format = FIO_OUTPUT_TERSE;
 			break;
 		case 'F':
+			if (!optarg) {
+				log_err("fio: missing --output-format argument\n");
+				exit_val = 1;
+				do_exit++;
+				break;
+			}
 			if (!strcmp(optarg, "minimal") ||
 			    !strcmp(optarg, "terse") ||
 			    !strcmp(optarg, "csv"))
@@ -1850,6 +1877,10 @@ int parse_cmd_line(int argc, char *argv[], int client_type)
 			do_exit++;
 			exit_val = fio_monotonic_clocktest();
 			break;
+		case 'G':
+			do_exit++;
+			exit_val = fio_crctest(optarg);
+			break;
 		case 'L': {
 			long long val;
 
